@@ -19,6 +19,8 @@ package config
 const (
 	ControllerName = "kourier"
 
+	EnvoyNodeID = "3scale-kourier-gateway"
+
 	InternalServiceName = "kourier-internal"
 	ExternalServiceName = "kourier"
 
@@ -35,6 +37,4 @@ const (
 	ExternalAuthzCluster     = "extAuthz"
 
 	KourierIngressClassName = "kourier.ingress.networking.knative.dev"
-	// Hack to force envoy reload
-	KourierHeaderRandom = "Kourier-random-header"
 )
