@@ -18,6 +18,7 @@ package generator
 
 import (
 	"errors"
+	"sync"
 
 	"go.uber.org/zap"
 	"knative.dev/net-kourier/pkg/envoy"
@@ -28,7 +29,6 @@ import (
 	route "github.com/envoyproxy/go-control-plane/envoy/api/v2/route"
 	"github.com/envoyproxy/go-control-plane/pkg/cache"
 	"github.com/google/uuid"
-	log "github.com/sirupsen/logrus"
 	kubeclient "k8s.io/client-go/kubernetes"
 	"knative.dev/serving/pkg/apis/networking/v1alpha1"
 )
@@ -36,6 +36,7 @@ import (
 var ErrDomainConflict = errors.New("ingress has a conflicting domain with another ingress")
 
 type Caches struct {
+	mu                  sync.Mutex
 	ingresses           map[string]*v1alpha1.Ingress
 	translatedIngresses map[string]*translatedIngress
 	clusters            *ClustersCache
@@ -46,23 +47,50 @@ type Caches struct {
 	logger              *zap.SugaredLogger
 }
 
-func NewCaches(logger *zap.SugaredLogger) *Caches {
-	return &Caches{
+func NewCaches(logger *zap.SugaredLogger, kubernetesClient kubeclient.Interface, extAuthz bool) (*Caches, error) {
+	c := &Caches{
 		ingresses:           make(map[string]*v1alpha1.Ingress),
 		translatedIngresses: make(map[string]*translatedIngress),
 		clusters:            newClustersCache(logger.Named("cluster-cache")),
 		clustersToIngress:   make(map[string][]string),
 		logger:              logger,
 	}
+	err := c.initConfig(kubernetesClient, extAuthz)
+	return c, err
+}
+
+func (caches *Caches) UpdateIngress(ingress *v1alpha1.Ingress, ingressTranslation *translatedIngress, kubeclient kubeclient.Interface) error {
+	// we hold a lock for Updating the ingress, to avoid another worker to generate an snapshot just when we have
+	// deleted the ingress before adding it.
+	caches.mu.Lock()
+	defer caches.mu.Unlock()
+
+	caches.deleteTranslatedIngress(ingress.Name, ingress.Namespace)
+
+	if err := caches.addTranslatedIngress(ingress, ingressTranslation); err != nil {
+		return err
+	}
+
+	return caches.setListeners(kubeclient)
+}
+
+func (caches *Caches) initConfig(kubernetesClient kubeclient.Interface, extAuthz bool) error {
+	if extAuthz {
+		extAuthZConfig := envoy.GetExternalAuthzConfig()
+		caches.addClusterForIngress(extAuthZConfig.Cluster, "__extAuthZCluster", "_internal")
+	}
+	caches.addStatusVirtualHost()
+	return caches.setListeners(kubernetesClient)
 }
 
 func (caches *Caches) GetIngress(ingressName, ingressNamespace string) *v1alpha1.Ingress {
+	caches.mu.Lock()
+	defer caches.mu.Unlock()
 	caches.logger.Debugf("getting ingress: %s/%s", ingressName, ingressNamespace)
 	return caches.ingresses[mapKey(ingressName, ingressNamespace)]
 }
 
 func (caches *Caches) validateIngress(translatedIngress *translatedIngress) error {
-
 	// We compare the Translated Ingress to current cached Virtualhosts, and look for any domain
 	// clashes. If there's one clashing domain, we reject the ingress.
 	localVhosts := caches.clusterLocalVirtualHosts()
@@ -87,7 +115,7 @@ func (caches *Caches) validateIngress(translatedIngress *translatedIngress) erro
 	return nil
 }
 
-func (caches *Caches) AddTranslatedIngress(ingress *v1alpha1.Ingress, translatedIngress *translatedIngress) error {
+func (caches *Caches) addTranslatedIngress(ingress *v1alpha1.Ingress, translatedIngress *translatedIngress) error {
 	caches.logger.Debugf("adding ingress: %s/%s", ingress.Name, ingress.Namespace)
 
 	if err := caches.validateIngress(translatedIngress); err != nil {
@@ -99,7 +127,7 @@ func (caches *Caches) AddTranslatedIngress(ingress *v1alpha1.Ingress, translated
 	caches.translatedIngresses[key] = translatedIngress
 
 	for _, cluster := range translatedIngress.clusters {
-		caches.AddClusterForIngress(cluster, ingress.Name, ingress.Namespace)
+		caches.addClusterForIngress(cluster, ingress.Name, ingress.Namespace)
 	}
 
 	return nil
@@ -110,12 +138,12 @@ func (caches *Caches) SetOnEvicted(f func(string, interface{})) {
 	caches.clusters.clusters.OnEvicted(f)
 }
 
-func (caches *Caches) AddStatusVirtualHost() {
+func (caches *Caches) addStatusVirtualHost() {
 	statusVirtualHost := statusVHost()
 	caches.statusVirtualHost = &statusVirtualHost
 }
 
-func (caches *Caches) SetListeners(kubeclient kubeclient.Interface) error {
+func (caches *Caches) setListeners(kubeclient kubeclient.Interface) error {
 	localVHosts := append(caches.clusterLocalVirtualHosts(), caches.statusVirtualHost)
 
 	listeners, err := listenersFromVirtualHosts(
@@ -136,8 +164,10 @@ func (caches *Caches) SetListeners(kubeclient kubeclient.Interface) error {
 }
 
 func (caches *Caches) ToEnvoySnapshot() (cache.Snapshot, error) {
-	caches.logger.Debugf("Preparing Envoy Snapshot")
+	caches.mu.Lock()
+	defer caches.mu.Unlock()
 
+	caches.logger.Debugf("Preparing Envoy Snapshot")
 	// Instead of sending the Routes, we send the RouteConfigs.
 	routes := make([]cache.Resource, len(caches.routeConfig))
 	for i := range caches.routeConfig {
@@ -160,7 +190,7 @@ func (caches *Caches) ToEnvoySnapshot() (cache.Snapshot, error) {
 	// to each envoy
 	snapshotVersion, err := caches.getNewSnapshotVersion()
 	if err != nil {
-		log.Errorf("Failed generating a new Snapshot version: %s", err)
+		caches.logger.Errorf("Failed generating a new Snapshot version: %s", err)
 		return cache.Snapshot{}, err
 	}
 
@@ -179,27 +209,11 @@ func (caches *Caches) ToEnvoySnapshot() (cache.Snapshot, error) {
 // time set in the "ClustersCache" struct.
 func (caches *Caches) DeleteIngressInfo(ingressName string, ingressNamespace string,
 	kubeclient kubeclient.Interface) error {
-	var err error
+	caches.mu.Lock()
+	defer caches.mu.Unlock()
+
 	caches.deleteTranslatedIngress(ingressName, ingressNamespace)
-
-	newExternalVirtualHosts := caches.externalVirtualHosts()
-	newClusterLocalVirtualHosts := caches.clusterLocalVirtualHosts()
-
-	statusVirtualHost := statusVHost()
-	newClusterLocalVirtualHosts = append(newClusterLocalVirtualHosts, &statusVirtualHost)
-
-	// We now need the cache in the listenersFromVirtualHosts.
-	caches.listeners, err = listenersFromVirtualHosts(
-		newExternalVirtualHosts,
-		newClusterLocalVirtualHosts,
-		caches.sniMatches(),
-		kubeclient,
-		caches,
-	)
-	if err != nil {
-		return err
-	}
-	return nil
+	return caches.setListeners(kubeclient)
 }
 
 func (caches *Caches) deleteTranslatedIngress(ingressName, ingressNamespace string) {
@@ -218,7 +232,7 @@ func (caches *Caches) deleteTranslatedIngress(ingressName, ingressNamespace stri
 	delete(caches.clustersToIngress, key)
 }
 
-func (caches *Caches) AddClusterForIngress(cluster *v2.Cluster, ingressName string, ingressNamespace string) {
+func (caches *Caches) addClusterForIngress(cluster *v2.Cluster, ingressName string, ingressNamespace string) {
 	caches.logger.Debugf("adding cluster %s for ingress %s/%s", cluster.Name, ingressName, ingressNamespace)
 
 	caches.clusters.set(cluster, ingressName, ingressNamespace)
